---
# Autogenerated nornir file
{% for dev in inventory %}
{{ dev.name }}:
    hostname: {{ dev.name }}
    groups:
        - {{ dev.operating_system }}
    data:
        mgmt_ip: {{ dev.mgmt_ip }}
        vendor: {{ dev.vendor|lower }}
        type: {{ dev.device_type }}
{% endfor %}