---
# Autogenerated Ansible hosts file
[ios]
{% for dev in inventory %}
{% if dev.operating_system == "ios" %}
{{ dev.name }} ansible_host={{ dev.mgmt_ip }}
{% else %}
{% endif %}
{% endfor %}

[nxos]
{% for dev in inventory %}
{% if dev.operating_system == "nxos" %}
{{ dev.name }} ansible_host={{ dev.mgmt_ip }}
{% else %}
{% endif %}
{% endfor %}

[iosxr]
{% for dev in inventory %}
{% if dev.operating_system == "iosxr" %}
{{ dev.name }} ansible_host={{ dev.mgmt_ip }}
{% else %}
{% endif %}
{% endfor %}

[junos]
{% for dev in inventory %}
{% if dev.operating_system == "junos" %}
{{ dev.name }} ansible_host={{ dev.mgmt_ip }}
{% else %}
{% endif %}
{% endfor %}

[eos]
{% for dev in inventory %}
{% if dev.operating_system == "eos" %}
{{ dev.name }} ansible_host={{ dev.mgmt_ip }}
{% else %}
{% endif %}
{% endfor %}

# Operating system specific vars

[ios:vars]
ansible_network_os=cisco.ios.ios
os=ios

[nxos:vars]
ansible_network_os=cisco.nxos.nxos
os=nxos

[iosxr:vars]
ansible_network_os=cisco.iosxr.iosxr
os=iosxr

[junos:vars]
ansible_network_os=junipernetworks.junos.junos
os=junos
ansible_connection=netconf

[eos:vars]
ansible_network_os=arista.eos.eos
os=eos

# All default vars

[all:vars]
ansible_connection=network_cli
{% raw %}
ansible_user="{{lookup('env','ANSIBLE_NET_UNAME')}}"
ansible_password="{{lookup('env','ANSIBLE_NET_PWORD')}}"
{% endraw %}