---
# Autogenerated nornir file
{% for grp in groups %}
{{ grp.name }}:
    platform: {{ grp.platform }}
    connection_options:
        scrapli:
            {% if grp.platform == "ios" %}
            platform: cisco_{{ grp.platform }}xe
            {% elif grp.platform == "iosxe" %} 
            platform: cisco_{{ grp.platform }}
            {% elif grp.platform == "junos" %}
            platform: juniper_{{ grp.platform }}
            {% elif grp.platform == "nxos" %}
            platform: cisco_{{ grp.platform }}
            {% elif grp.platform == "nxos_ssh" %}
            platform: cisco_nxos
            {% elif grp.platform == "eos" %}
            platform: arista_{{ grp.platform }}
            {% else %}
            platform: {{ grp.platform }}
            {% endif %}
            port: {{ grp.port }}
            extras:
                auth_strict_key: {{ grp.auth_strict_key }}
{% endfor %}